/*===============================================================================
Title: dwDueCC3000.ino
Description: deviceWISE Arduino DUE with CC3000 WiFi Sample Program

Copyright (c)2015 Telit.
http://www.ilstechnology.com
http://www.devicewise.com


Legal Disclaimer:

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Neither the name of ILS Technology nor Telit nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

===================================================================================*/

#include <TICC3000.h>
#include <ccspi.h>
#include <SPI.h>
#include <string.h>

#include "utility/debug.h"

#include <dwArduinoCC3000OpenClient.h>

// These are the interrupt and control pins
#define TICC3000_IRQ   3  // MUST be an interrupt pin!
// These can be any two pins
#define TICC3000_VBAT  5
#define TICC3000_CS    10
// Use hardware SPI for the remaining pins
// On an UNO, SCK = 13, MISO = 12, and MOSI = 11
TICC3000 cc3000 = TICC3000(TICC3000_CS, TICC3000_IRQ, TICC3000_VBAT, SPI_CLOCK_DIVIDER); // you can change this clock speed

#define WLAN_SSID       "Telit_GUEST"        // cannot be longer than 32 characters!
#define WLAN_PW         "qwerty123456"

// Security can be WLAN_SEC_UNSEC, WLAN_SEC_WEP, WLAN_SEC_WPA or WLAN_SEC_WPA2
#define WLAN_SECURITY   WLAN_SEC_WPA2

// Authentication/Registration Details      
#define DWOPEN_APPTOKEN    "Xx7PIfDJQzhtVcdP"      //Application Token
#define DWOPEN_SERVER      "api.devicewise.com"    //deviceWISE server
#define DWOPEN_PORT        1883                    //MQTT Port

//ILST Headquarters - GPS Detail...
#define ILST_Latitude      "26.394919"
#define ILST_Longitude     "-80.112808"
#define ILST_Altitude      "3.0"
#define ILST_Speed         "0.0"
#define ILST_Heading       "0.0"

char dwThingKey[40];
byte mac[6] = {0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5};

void callback(char* topic, uint8_t* payload, unsigned int length);

TICC3000_Client c;
dwOpenClient dwClient((char*) DWOPEN_SERVER, DWOPEN_PORT, callback, &c);

#define RED_LED           4               //D4 - RED LED
#define BUZZER_PIN        2               //D2 - Buzzer
#define ANGLE_PIN         A0              //A0 - Angle/Rotary Sensor  

bool displayConnectionDetails(void)
{
  uint32_t ipAddress, netmask, gateway, dhcpserv, dnsserv;
  
  if(!cc3000.getIPAddress(&ipAddress, &netmask, &gateway, &dhcpserv, &dnsserv))
  {
    Serial.println(F("Unable to retrieve the IP Address!\r\n"));
    return false;
  }
  else
  {
    Serial.print(F("\nIP Addr: ")); cc3000.printIPdotsRev(ipAddress);
    Serial.print(F("\nNetmask: ")); cc3000.printIPdotsRev(netmask);
    Serial.print(F("\nGateway: ")); cc3000.printIPdotsRev(gateway);
    Serial.print(F("\nDHCPsrv: ")); cc3000.printIPdotsRev(dhcpserv);
    Serial.print(F("\nDNSserv: ")); cc3000.printIPdotsRev(dnsserv);
    Serial.println();
    return true;
  }
}

void setup() 
{
  int i;
  
  Serial.begin(9600);
  while( !Serial ) delay( 100 );  
  
  for(i = 5; i > 0; i-- )
  {
    Serial.print("Ready to Execute... Countdown = ");
    Serial.println(i); 
    delay( 1000 );
  }
  
  Serial.println(F("Arduino DUE - dwOpen IoT Client Device"));

  Serial.print(F("Free RAM: ")); Serial.println(getFreeRam(), DEC);
  
  Serial.print(F("Attempting to connect to Network named: ")); Serial.println(WLAN_SSID); 

  /* Initialise the module */
  Serial.println(F("\nInitializing Wi-Fi..."));
  if (!cc3000.begin())
  {
    Serial.println(F("Couldn't begin()! Check your wiring?"));
    while(1);
  }
  
  if (!cc3000.connectToAP(WLAN_SSID, WLAN_PW, WLAN_SECURITY)) 
  {
    Serial.println(F("Wi-Fi Failed!"));
    while(1);
  }
   
  Serial.println(F("Wi-Fi Connected!"));
  
  /* Wait for DHCP to complete */
  Serial.println(F("Requesting DHCP..."));
  while (!cc3000.checkDHCP())
  {
    delay(100); // ToDo: Insert a DHCP timeout!
  }  

  /* Display the IP address DNS, Gateway, etc. */  
  while( !displayConnectionDetails() ) 
  {
    delay(1000);
  } 
  
  //Get my MAC address
  // WiFi.macAddress(mac);
  cc3000.getMacAddress(mac);

  //Format the ThingKey for this run using our MAC
  sprintf(dwThingKey, "ArduinoDUE-%x%x%x%x%x%x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);  
  Serial.print("\nMy Thing Key: ");  Serial.println( dwThingKey );   
  
  //Configure RED LEDs
  pinMode(RED_LED, OUTPUT);
  digitalWrite(RED_LED, LOW);  

  //Configure Buzzer
  pinMode(BUZZER_PIN, OUTPUT);
  digitalWrite(BUZZER_PIN, LOW);
}

void loop() 
{
  int rc = -1;
  char szText[128];
  char szPropValue[16];
  static int LastRotary = -1;

  //Already connected to the cloud?
  if( !dwClient.connected() )
  {
      //Not connected, try connecting to the cloud.
      
      Serial.print("\nConnecting to: "); Serial.print(DWOPEN_SERVER); Serial.print(" with client id: "); Serial.println(dwThingKey);
  
      dwClient.connect((char*) dwThingKey, (char*) dwThingKey, (char*) DWOPEN_APPTOKEN);

      //Did we connect to the cloud?
      if( dwClient.connected() )
      {
          //Yes, we're connected to the cloud.
          Serial.println("Connected to m2mAIR Cloud...");      
          dwClient.dwSleep(1000);   
                
          //Serial.println("Subscribing to Topic 'reply'...");
          dwClient.subscribe((char *) MQTT_REPLY);
          dwClient.dwSleep(1000);       
                
          //Serial.println("Subscribing to Topic 'notify'...");
          dwClient.subscribe((char *) MQTT_NOTIFY);
          dwClient.dwSleep(1000);      
           
          //Publish Log Message... Connected
          Serial.println("\ndwLaunchPad- Log Publish In Process..." );
          rc = dwClient.dwLogPublish( (char*) dwThingKey, (char*) "Arduino DUE IoT Application Started...");      
          Serial.print("Publish Log Message return code : "); Serial.println(rc);       
          dwClient.dwSleep(1000);      
    
     
          //Publish Location Message...
          Serial.println("\ndwOpen API - Location Publish In Process..." );      
          rc = dwClient.dwLocationPublish( dwThingKey, (char *) ILST_Latitude, (char *) ILST_Longitude, (char *) ILST_Altitude, (char *) ILST_Speed, (char *) ILST_Heading );            
          Serial.print("Publish Location return code : "); Serial.println(rc);          
          dwClient.dwSleep(1000);      
          
          //Publish Attribute...
          Serial.println("\ndwOpen API - Attribute Publish In Process..." );    
          sprintf(szPropValue, "%x%x%x%x%x%x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);     
          rc = dwClient.dwAttributePublish( dwThingKey, (char *) "MAC", szPropValue );    
          
          Serial.print("Publish Location return code : "); Serial.println(rc);    
          dwClient.dwSleep(1000);            
       
      }    
       
  }


  //Are we connected to the cloud?
  if( dwClient.connected() )   
  {  
        //Yes, Read sensor value from potentiometer
        int RotaryValue = analogRead(ANGLE_PIN); 
  
        //Did the Rotary Value change?
        if (RotaryValue != LastRotary) 
        {
            //Yes, Publish new rotary value
            Serial.print(dwThingKey); Serial.print(", RotaryValue:"); Serial.println(RotaryValue);         
            sprintf(szPropValue,"%d", RotaryValue);             
            
            Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Rotary Property Publish...");
            rc = dwClient.dwPropertyPublish( dwThingKey, (char *) "Rotary", szPropValue );     
            Serial.print(dwThingKey); Serial.print(",dwOpen API - Rotary Property Publish Returned... rc = "); Serial.println(rc);
            
            //------------------------------------------------
            //-- Alarm Publish
            //------------------------------------------------
  
            int alarmState;
            char alarmMsg[40];
            
            // Various simulated alarm status...
            if (RotaryValue < 200)
            {
              alarmState = 0;
              strcpy( alarmMsg, "Temp - OK" );
              Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Alarm Publish Request... OK");    
            }
            else if (RotaryValue < 900)
            {
                alarmState = 1;
                strcpy( alarmMsg, "Temp - Warning");
                Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Alarm Publish Request... Warning");
            }
            else
            {
                alarmState = 2;
                strcpy(alarmMsg, "Temp - HIGH" );
                Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Alarm Publish Request... HIGH");
            }
                  
            rc = dwClient.dwAlarmPublish( dwThingKey, (char *) "alarm1", alarmState, alarmMsg );
            Serial.print(dwThingKey); Serial.print(",dwOpen API - Alarm Publish Returned... rc = "); Serial.println(rc);            
            
            //Save our last rotary reading  
            LastRotary = RotaryValue;
        }             
  }
  else
  {
      Serial.println("Not connected to the cloud yet...");
  }

  Serial.println("Sleeping for 5 Seconds..." );  
  dwClient.dwSleep(5000);
  
}


void ExtractKeyValue( char *haystack, char *Key, char *ResultValue )
{
  char *KeyPtr;
  char *KeyValue;
  char *KeyValueEnd;
  int   KeyValueLen;
  
  char needleTmp[40];

   //Serial.print("ExtractKeyValue: ");Serial.println(Key);
   //Serial.print("ExtractHaystack: ");Serial.println(haystack);
              
    strcpy( needleTmp, "\"");
    strcat( needleTmp, Key);
    strcat( needleTmp, "\":\"");

    KeyPtr = strstr(haystack, needleTmp );
      
    if( KeyPtr != NULL) 
    {   
        KeyValue = KeyPtr + strlen(needleTmp);
        KeyValueEnd = KeyValue;
    
        while( *KeyValueEnd != '\"')
        {
            KeyValueEnd++;
        }
    
        KeyValueLen = KeyValueEnd - KeyValue;
        strncpy( ResultValue, KeyValue, KeyValueLen );
        *(ResultValue + KeyValueLen ) = 0;
    }
    else
    {
        strcpy( ResultValue, "" );
    }
    
    //Serial.print("ExtractKeyValue Result: ");Serial.println(ResultValue);
}
        
// Sets the LED accordingly
void setLED( int LEDPin, char *LEDStatus )
{

    if ( strcmp(LEDStatus, "1") == 0 )
    {        
        //Serial.println("LED On!!!");          
        digitalWrite(LEDPin, HIGH);
    }
    else
    {
        //Serial.println("LED Off!!!");          
        digitalWrite(LEDPin, LOW);          
    }
   
}

// HandlesBuzzer Beep
void handleBuzzer( char *Value )
{
  int Duration = atoi( Value );

  Serial.print("in handleBuzzer: ");  Serial.println( Value );   

  digitalWrite(BUZZER_PIN, HIGH);   
  delay(Duration);
  digitalWrite(BUZZER_PIN, LOW);   
}

//TR50 JSON Sample Request/Response
#define MAILBOX_IN_PROGRESS_ACK  "{\"999-999\": { \"command\": \"mailbox.check\", \"params\": { \"autoComplete\": true, \"limit\": 0 }}}"
#define MAILBOX_REQ_RECEIVED     "{\"0000123\": { \"command\": \"mailbox.update\", \"params\": {\"id\": \"%s\", \"msg\": \"Executing...\"}}}"
#define MAILBOX_REQ_ACK          "{\"0000124\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"%s\", \"errorCode\": \"0\", \"errorMessage\": \"OK\"}}}"

// The function is called when an MQTT message arrives
void callback(char *topic, uint8_t *payload, unsigned int length) 
{
  int i;
  char jsonText[1024];
  char Value[10];   
  char msgID[50];

  //Serial.println("Callback - MQTT Message Arrived...");
  //Serial.print("Topic: ");  Serial.println( topic ); 
  
  //Serial.print("Payload Length: ");  Serial.println( length ); 
  //Serial.print("Payload: ");  Serial.println( (char *) payload );
  
  if(strstr(topic, MQTT_NOTIFY) != NULL) 
  {  
      //Serial.println("Notification - Mailbox Activity... Check Mailbox");
      dwClient.publish( (char *) "api", (char *) MAILBOX_IN_PROGRESS_ACK );    
  }
  else if( (strstr(topic, MQTT_REPLY) != NULL) )
  {
      if ((strstr((char *)payload, (char *) "999-999") != NULL)) 
      {     
        //Serial.println("Reply - Mailbox Check Response... Mailbox Request Received");   
        ExtractKeyValue( (char *) payload, (char *) "id", msgID ); 
        
        sprintf( jsonText, MAILBOX_REQ_RECEIVED, msgID );  
        dwClient.publish( (char *) "api", jsonText );
  
        //Set LED accordingly                 
        ExtractKeyValue( (char *) payload, (char *) "varled", Value );                
        if ( strlen(Value) ) setLED(RED_LED, Value);

        //Handle if Buzzer
        ExtractKeyValue( (char *) payload, (char *) "duration", Value );                
        if ( strlen(Value) ) handleBuzzer(Value);        
          
        //Serial.println("Processing Complete - Send Mailbox Acknowledge...");
        sprintf( jsonText, MAILBOX_REQ_ACK, msgID );   
        dwClient.publish( (char *) "api", jsonText );  
      } 
    
  }
  
}



